package com.ip2.api.controller;

import com.google.gson.Gson;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.google.gson.reflect.TypeToken;
import com.ip2.api.Ip2Constants;
import com.ip2.api.bo.SearchParam;
import com.ip2.api.util.Ip2HttpUtil;
import org.apache.log4j.Logger;

import java.lang.reflect.Type;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

public class SearchParamController {

    private static Logger logger = Logger.getLogger(SearchParamController.class);

    private Ip2HttpUtil ip2HttpUtil = new Ip2HttpUtil();

    public List<SearchParam> getSearchParams(int projectId) {

        List<SearchParam> searchParamList = new ArrayList<>();

        try {

            URI uri = ip2HttpUtil.getURIBuilder().setPath(Ip2Constants.getInstance().getSearchParamPath())
                    .setParameter("pid", String.valueOf(projectId))
                    .build();

            String jsonString = ip2HttpUtil.getUriResponse(uri);

            Type searchParam = new TypeToken<List<SearchParam>>() {
            }.getType();

            searchParamList = new Gson().fromJson(jsonString, searchParam);

            logger.debug("searchParamList size :: " + searchParamList.size());

        } catch (URISyntaxException e) {
            e.printStackTrace();
        }

        return searchParamList;
    }

    public Integer createNewSearch(String projectId,
                               String sampleName,
                               String searchParamId,
                               String experimentDate,
                               String instrumentId) {

        Integer searchId = null;

        try {
            URI uri = ip2HttpUtil.getURIBuilder().setPath(Ip2Constants.getInstance().getCreateSearchPath())
                    .setParameter("pid", projectId)
                    .setParameter("searchParamId", searchParamId)
                    .setParameter("sampleName", sampleName)
                    .setParameter("experimentDate", experimentDate)
                    .setParameter("instrumentId", instrumentId)
                    .build();

            String jsonString = ip2HttpUtil.getPostUriResponse(uri);

            JsonObject jsonObject = (JsonObject) JsonParser.parseString(jsonString);

            searchId = jsonObject.get("searchId").getAsInt();

            logger.debug("New Search ID :" + searchId);

        } catch (URISyntaxException e) {
            e.printStackTrace();
        }

        return searchId;
    }

    public static void main(String[] args) {

        AccountController accountController = new AccountController();
        accountController.authenticate("ip2test", "ip2test321");

        SearchParamController searchParamController = new SearchParamController();
        searchParamController.getSearchParams(5343).forEach(searchParam -> System.out.println(searchParam.toString()));

        searchParamController.createNewSearch("5343",
                "TestSampleName", "123",
                "2020-09-06", "123");
    }
}
