package com.ip2.paser;
import com.ip2.paser.util.PaserConstants;
import org.apache.log4j.Logger;
import java.io.*;

public class PaserInstance {
    private String uniqueId;

    private String pathToPtxFile;
    private String pathToSearchXmlFile;

    private Ms2SpectraConsumer spectraConsumer;
    private PrecursorConsumer precursorConsumer;

    private static Logger initLogger = Logger.getLogger(InitializationConsumer.class);
    private Logger paserLogger = Logger.getLogger(PaserInstance.class);
    private File outputDir;

    private PaserConstants paserConstants;

    public PaserInstance(String id){

        paserConstants = PaserConstants.getInstance();
        uniqueId = id;

        //  IP2 connection (VJAY CODE)
        //DbSearch dbSearch = searchApiController.getDbSearch(searchId);
        //String spectraPath = dbSearch.getExperiemntHomeFolder() + File.separator + "spectra";
        //String searchFolder = dbSearch.getSearchResultPath();

        pathToPtxFile = paserConstants.getPathToPtxFile();
        pathToSearchXmlFile = paserConstants.getPathToSearchXmlFile();
        paserLogger.info(" pathToPtxFile: " + pathToPtxFile);
        paserLogger.info(" pathToSearchXmlFile: " + pathToSearchXmlFile);

        // create dir for sqt and ms2 filers
        String outputPath = paserConstants.getDataOutPath() + File.separator + uniqueId;
        outputDir = new File(outputPath);
        outputDir.mkdir();


        // pass context and array ofGpuWorkspaces to reactive consumer.
        spectraConsumer = new Ms2SpectraConsumer(pathToPtxFile, pathToSearchXmlFile, uniqueId);
        ///precursorConsumer = new PrecursorConsumer(uniqueId);
    }

    public void run(){
        //start consumers for Spectra

        //precursorConsumer.consumeMessages();
        spectraConsumer.consumeMessages();
    }

    public void free(){
        spectraConsumer.free();
        //precursorConsumer.free();
        mergeSqtFiles();
    }

    public void mergeSqtFiles(){

        // Merge SQT Files into one
        File[] listOfFiles = outputDir.listFiles();
        String mergedFilePath = outputDir.getPath() + File.separator + "Realtime_" + uniqueId + ".sqt";
        BufferedWriter bw = null;

        try {
            bw = new BufferedWriter(new FileWriter(mergedFilePath));
        }catch (IOException e) {
            paserLogger.error(e);
            paserLogger.error(e.getStackTrace());
        }

        if (listOfFiles == null) {
            return;
        }

        for(File f :listOfFiles){
            //File is SQT
            if(f.getName().contains(".sqt")){
                try {
                    BufferedReader br = new BufferedReader(new FileReader(f));
                    String st;
                    while ((st = br.readLine()) != null) {
                        bw.write(st);
                        bw.newLine();
                    }
                    bw.flush();
                    f.delete();
                } catch( Exception e){
                    paserLogger.error(e);
                    paserLogger.error(e.getStackTrace());
                }
            }
        }
    }
}
