package com.ip2.paser;//import util.properties packages
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.DoubleBuffer;
import java.sql.*;
import java.time.Duration;
import java.util.ArrayList;
import java.util.BitSet;
import java.util.Collections;
import java.util.Properties;
import java.util.concurrent.ExecutionException;

//import simple producer packages
import org.apache.kafka.clients.consumer.Consumer;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.clients.producer.*;

//import KafkaProducer packages

//import ProducerRecord packages
import org.apache.kafka.common.serialization.ByteArrayDeserializer;
import org.apache.kafka.common.serialization.ByteArraySerializer;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.apache.kafka.common.serialization.StringSerializer;

public class PrecursorIdentification {

    private final static String TOPIC = "prec";
    private final static String BOOTSTRAP_SERVER = "172.29.80.71:9092";

    private final static int SIZE_OF_DATABASE = 6500000;
    private final static int PPM_DIVISOR = 1000000;
    private final static double PROTON_MASS = 1.00727826;

    String fafstaFilePath = "";
    int maxDatabaseValue = 0;
    int minDatabaseValue = SIZE_OF_DATABASE;

    private Connection specDbConnection;
    private int ppm = 40;
    public byte[] database;
    public BitSet database_bits;

    int numberOfPrecursors = 0;
    File fileLocation;

    public PrecursorIdentification(String fafstaFilePath) throws SQLException {
        this.fafstaFilePath = fafstaFilePath;
        database = new byte[SIZE_OF_DATABASE];
        database_bits = new BitSet(SIZE_OF_DATABASE);
        fillBooleanDatabaseFromFafstaFile();
    }

    private void fillBooleanDatabaseFromFafstaFile() {
        File filePath = new File(fafstaFilePath);
        File[] listingAllFiles = filePath.listFiles();

        for (File file : listingAllFiles) {
            fileLocation = findFileswithIndExtension(file);
            int count = 0;
            if (fileLocation != null) {
                System.out.println("Loading " + file.getName() + " into database");

                //Get Database connection to idx file
                try{
                    specDbConnection = DriverManager.getConnection("jdbc:sqlite:" + file.getAbsolutePath());
                } catch (SQLException ex){
                    System.out.println(ex);
                    continue;
                }

                try (Statement stmt  = specDbConnection.createStatement();
                     ResultSet rs    = stmt.executeQuery("SELECT * FROM blazmass_sequences")
                ){

                    while (rs.next()) {

                        //System.out.println(count);
                        int massKey = rs.getInt("precursor_mass_key");
                        byte[] arr = rs.getBytes("data");
                        ByteBuffer wrapped = ByteBuffer.wrap(arr).order(ByteOrder.LITTLE_ENDIAN); // big-endian by default

                        for(int i = 0; i<arr.length; i += 16){

                            float precursorMass = wrapped.getFloat();       int seqOffset = wrapped.getInt();
                            int seqLength = wrapped.getInt();               int proteinId = wrapped.getInt();

                            insertPeptideIntoDataBase((int)(precursorMass * 1000));
                            insertPeptideIntoDataBase((int)((precursorMass + PROTON_MASS*1) * 1000));
                            insertPeptideIntoDataBase((int)((precursorMass + PROTON_MASS*2) * 1000));
                            insertPeptideIntoDataBase((int)((precursorMass + PROTON_MASS*3) * 1000));

                            //Skip peptide sequence
                            while(true){
                                if(wrapped.getInt() == Integer.MAX_VALUE){
                                    i+=4;
                                    break;
                                }
                                i+=4;
                            }
                        }
                    }
                } catch (SQLException ex) {
                    ex.printStackTrace();
                }
            }
        }
        System.out.println("    Number of precursors loaded: " + numberOfPrecursors);
    }

    public static File findFileswithIndExtension(File file) {

        if(file.getName().toLowerCase().endsWith("idx")) {
            return file;
        }
        return null;
    }

    public synchronized void insertPeptideIntoDataBase(double massKey){

        if(massKey > maxDatabaseValue){
            maxDatabaseValue = (int) massKey;
        }
        if(massKey < minDatabaseValue){
            minDatabaseValue = (int) massKey;
        }

        double tolerance = ppm*massKey/(PPM_DIVISOR);
        //System.out.println("Mass Key: " + massKey + ", Tolerance: " + tolerance);
        //int tolerance = (int) (massKey*0.00004);

        int start = (int) (massKey - tolerance);
        int end = (int) (massKey + tolerance);
        //System.out.println("Mass: " + massKey + "   |   Range: " + (end - start));
        if (start < 0) {
            start = 0;
        }
        if (end > database.length - 1) {
            end = database.length - 1;
        }
        for(int i = start; i<end; i++){
            if(database[i] != 1) {
                database[i] = 1;
                //database_bits.set(i);
                //numberOfIndexesTrueInDatabase++;
            }
        }
    }

    public static Producer<String, byte[]> createProducer() {
        Properties props = new Properties();
        props.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, BOOTSTRAP_SERVER);
        props.put(ProducerConfig.CLIENT_ID_CONFIG, "PrecursorDatabase");
        props.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class.getName());
        props.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, ByteArraySerializer.class.getName());
        props.put(ProducerConfig.MAX_REQUEST_SIZE_CONFIG, 1024 * 1024 * 10);
        return new KafkaProducer<>(props);
    }

    public void runProducer(){

        Producer<String, byte[]> producer = createProducer();

        //byte[] bytes = new byte[(database_bits.length() + 7) / 8];

        try {
            final ProducerRecord<String, byte[]> record =
                    new ProducerRecord<>("PrecursorDatabase", "1234", database);
            RecordMetadata metadata = producer.send((ProducerRecord<String, byte[]>) record).get();
            System.out.printf("sent record(key=%s value=%s) ",
                    record.key(), record.value());


        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (ExecutionException e) {
            e.printStackTrace();
        } finally {
            producer.flush();
            //producer.close();
        }

    }
}
