package com.ip2.paser;

import blazmass.dbindex.DBIndexer;
import edu.scripps.pms.mspid.*;
import org.apache.log4j.Logger;
import org.jdom.JDOMException;
import java.io.IOException;
import static edu.scripps.pms.mspid.GPUSpectraWorkSpace.generateWorkSpace;
import static edu.scripps.pms.mspid.Ms2FileThread.createDB;
import static jcuda.driver.JCudaDriver.*;

/**
 * ProlucidWorkSpaceManager sets up Spectra Workspace objects for GPU search
 */
public class ProlucidWorkSpaceController {

    private String pathToPtxFile;
    private String pathToSearchXmlFile;
    private int coreCount;

    private DistributionCalculator dc;
    private CudaBinomial cudaBinomial;
    private CudaDeviceContext context;
    private GPUSpectraWorkSpace gpuSpectraWorkSpaces[];
    private DBIndexer dbList[];

    private Logger logger = Logger.getLogger(PaserInstance.class);


    /**
     * Creates a number of gpu WorkSpaces and places them in an array.
     * @param pathToPtx
     * @param pathToSearchXml
     * @throws Exception
     */
    public ProlucidWorkSpaceController(String pathToPtx, String pathToSearchXml, int workspaceThreads) throws Exception {
        pathToPtxFile = pathToPtx;
        pathToSearchXmlFile = pathToSearchXml;
        coreCount = workspaceThreads;

        gpuSpectraWorkSpaces = new GPUSpectraWorkSpace[coreCount];
        dbList = new DBIndexer[coreCount];
        initializeGPUWorkSpaces();
    }

    private void initializeGPUWorkSpaces() throws IOException, JDOMException {

        // Initialize CudaDeviceContext
        context = new CudaDeviceContext(0,false);
        CudaDeviceContext.SINGLE_SCAN_MODE_PTX = pathToPtxFile;
        context.prepare();

        // "whatever" - Titus
        if(dc == null) {
            dc = new DistributionCalculator();
        }

        SearchParams sp = new SearchParams("",pathToSearchXmlFile);
        MassCalculator mc = new MassCalculator(sp);
        //DBIndexer db = createDB(sp);
        cudaBinomial = new CudaBinomial(context.getSetBinomial(),mc);
        cudaBinomial.assignBinomials(null);

        for(int i = 0; i < gpuSpectraWorkSpaces.length; i++){
            try {
                DBIndexer dbi = createDB(sp);
                dbList[i] = dbi;
                gpuSpectraWorkSpaces[i] = generateWorkSpace(sp, context, false, null, dbi);

            } catch (Exception e){
                logger.error("Exception found generating the workspace");
                logger.error(e);
                // TODO: Handle this error
            }
        }
    }

    public GPUSpectraWorkSpace[] getGpuSpectraWorkSpaces(){
        return gpuSpectraWorkSpaces;
    }

    public CudaDeviceContext getContext() {
        return context;
    }

    // TODO: Not sure if all of this is needed. May be redundant. Also errors involved when
    //       deleted too quickly after init
    public void free(){
        for (GPUSpectraWorkSpace gpu : gpuSpectraWorkSpaces) {
            if(gpu != null) {
                try {
                    logger.info("free gpuWrkSpc");
                    gpu.free();
                } catch (Exception e){
                    logger.error("Exception freeing gpu workspace");
                    logger.error(e);
                    logger.error(e.getStackTrace());
                }
            }
        }

        for (DBIndexer dbi : dbList) {
            logger.info("free dbi");
            dbi.close();
        }

        logger.info("Free binomial and workspaces");
        if(cudaBinomial != null){
            try {
                cudaBinomial.free();
            } catch (Exception e) {
                logger.error("Exception freeing cudaBinomial");
                logger.error(e);
                logger.error(e.getStackTrace());
            }
        }

        // TODO: free context for eacah gpu thread?
        logger.info("Free context");
        if(context != null){
            cuCtxDestroy(context.getContext());
        }
    }
}
