package com.ip2.paser.UnitTests;
import com.ip2.paser.InitializationConsumer;
import com.ip2.paser.util.PaserConstants;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.apache.kafka.common.serialization.StringSerializer;
import org.json.simple.JSONObject;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.assertEquals;

//import util.properties packages
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.Properties;
//import simple producer packages
import org.apache.kafka.clients.producer.Producer;
//import KafkaProducer packages
import org.apache.kafka.clients.producer.KafkaProducer;
//import ProducerRecord packages
import org.apache.kafka.clients.producer.ProducerRecord;
import org.junit.runner.JUnitCore;
import org.junit.runner.Result;
import org.junit.runner.notification.Failure;


public class InitTests {

    InitializationConsumer rtsah;
    testProducer simp;
    testConsumer con;

    private PaserConstants paserConstants;

    public InitTests(){
        paserConstants = PaserConstants.getInstance();
    }

    private String createJsonMessage(String id, String searchid){

        String uniqueID = id;
        String searchId = "12345";

        String initMessage;
        JSONObject json = new JSONObject();
        json.put("id",uniqueID);
        json.put("search_id",searchId);
        initMessage = json.toString();
        System.out.println(initMessage);
        return initMessage;
    }

    private String getUniqueId(String testCase){
        Date dNow = new Date();
        SimpleDateFormat ft = new SimpleDateFormat("yyMMddhhmmssMs");
        String datetime = ft.format(dNow);

        return testCase + datetime;
    }

    @Before
    public void setup() {
        System.out.println("Setup");
        //Unit Tests for Initialization
        rtsah = new InitializationConsumer();
        rtsah.consumeMessages();
        simp = new testProducer(paserConstants.getInitializationTopic());
        con = new testConsumer(paserConstants.getInitializationTopic());

    }

    @After
    public void teardown() {
        System.out.println("teardown");

        rtsah.free();
        simp.free();
        con.free();
    }

    @Test
    public void testInitCompleted() {
        //Unit Tests for Initialization

        System.out.println("begin test: testInitCompleted");

        String uniqueID = getUniqueId("testInitCompleted");
        String searchId = "12345";
        String message = createJsonMessage(uniqueID, searchId);

        simp.sendMessage(paserConstants.getInitStartKey(), message);
        boolean result = con.isInitializationComplete(uniqueID);

        System.out.println(result);
        Assert.assertTrue(result);

    }

    @Test
    public void testWrongInitValue() {
        //Unit Tests for Initialization
        System.out.println("begin test: testWrongInitValue");


        String uniqueID = getUniqueId("testWrongInitValue");
        String searchId = "12345";
        String message = "This is an incorrect message";

        simp.sendMessage(paserConstants.getInitStartKey(), message);
        boolean result = con.isInitializationFailed("Init Not Complete: Parse start args failed");

        System.out.println(result);
        Assert.assertTrue(result);
    }

    @Test
    public void testWrongInitKey() {
        //Unit Tests for Initialization
        System.out.println("begin test: testWrongInitKey");

        String uniqueID = getUniqueId("testWrongInitKey");
        String searchId = "12345";
        String message = createJsonMessage(uniqueID, searchId);

        simp.sendMessage("wrongValue", message);
        boolean result = con.isInitializationComplete(uniqueID);

        System.out.println(!result);
        Assert.assertFalse(result);
    }

//    @Test
//    public void testRemovePreviousPaserInstance() throws InterruptedException {
//        //Unit Tests for Initialization
//        System.out.println("begin test: testRemovePreviousPaserInstance");
//
//        String uniqueID = getUniqueId("testRemovePreviousPaserInstance");
//        String searchId = "12345";
//
//        String lastID = uniqueID;
//        for (int i = 0; i < 5; i++){
//            simp.sendMessage(paserConstants.getInitStartKey(), createJsonMessage(uniqueID + i, searchId));
//            Thread.sleep(500);
//            lastID = uniqueID + i;
//        }
//
//        boolean result = con.isInitializationComplete(lastID);
//
//        System.out.println(result);
//        Assert.assertTrue(result);
//    }

}

class MyTestRunner {
    public static void main(String[] args) {
        Result result = JUnitCore.runClasses(InitTests.class);
        for (Failure failure : result.getFailures()) {
            System.out.println(failure.toString());
        }
    }
}


class testProducer {

    String kafkaTopic;
    Producer<String, String> producer;

    public testProducer(String topic){
        kafkaTopic = topic;
        setup();
    }

    private void setup(){
        // create instance for properties to access producer configs
        Properties props = new Properties();

        //Assign localhost id
        props.put("bootstrap.servers","localhost:9092");

        //Set acknowledgements for producer requests.
        props.put("acks", "all");

        //If the request fails, the producer can automatically retry,
        props.put("retries", 0);

        //Specify buffer size in config
        props.put("batch.size", 16384);

        //Reduce the no of requests less than 0
        props.put("linger.ms", 1);

        //The buffer.memory controls the total amount of memory available to the producer for buffering.
        props.put("buffer.memory", 33554432);

        props.put(ProducerConfig.CLIENT_ID_CONFIG, "Initialization-tester");

        props.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG,
                StringSerializer.class.getName());
        props.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG,
                StringSerializer.class.getName());

        producer = new KafkaProducer<String, String>(props);
    }

    public void sendMessage(String Key, String messsage){
        producer.send(new ProducerRecord<String, String>(kafkaTopic,
                Key, messsage));
    }

    public void free(){
        producer.close();
    }

}

class testConsumer {

    String topicName;
    KafkaConsumer<String, String> consumer;

    public testConsumer(String topic){
        topicName = topic;
        setup();
        consumer.subscribe(Arrays.asList(topicName));
    }

    private void setup(){
        Properties props = new Properties();

        props.put("bootstrap.servers", "localhost:9092");
        props.put("group.id", "test");
        props.put("enable.auto.commit", "true");
        props.put("auto.commit.interval.ms", "1000");
        props.put("session.timeout.ms", "30000");
        props.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG,
                StringDeserializer.class.getName());
        props.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG,
                StringDeserializer.class.getName());
        props.put(ConsumerConfig.AUTO_OFFSET_RESET_CONFIG, "latest");

        consumer = new KafkaConsumer<String, String>(props);

    }
    public boolean isInitializationComplete(String id) {

        for(int i =0; i < 10; i++) {
            ConsumerRecords<String, String> records = consumer.poll(1000);

            for (ConsumerRecord<String, String> record : records) {

                if (record.value().equals(id) && record.key().equals("Completed")) {
                    System.out.printf("offset = %d, key = %s, value = %s\n",
                            record.offset(), record.key(), record.value());
                    return true;
                }
            }
        }

        return false;

    }

    public boolean isInitializationFailed(String message) {

        for(int i =0; i < 10; i++) {
            ConsumerRecords<String, String> records = consumer.poll(1000);

            for (ConsumerRecord<String, String> record : records) {

                if (record.key().equals("Failed") && record.value().equals(message)) {
                    System.out.printf("offset = %d, key = %s, value = %s\n",
                            record.offset(), record.key(), record.value());
                    return true;
                }
            }
        }

        return false;

    }

    public void free(){
        consumer.close();
    }

}