package com.ip2.paser.util;

import com.ip2.api.Ip2Constants;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;
import org.apache.commons.io.FilenameUtils;

public class PaserConstants {

    private static PaserConstants instance = null;

    private String kafkaIP = "localhost:9092";
    private int NumberOfThreadsForGPUSearch = 8;
    private String dataOutPath = "data/";
    private String pathToPtxFile = "";
    private String pathToSearchXmlFile = "";

    private String InitStartKey = "Start";
    private String InitEndKey = "End";
    private String InitCompletedKey = "Completed";
    private String InitFailedKey = "Failed";

    private String InitializationTopic = "Initialization";
    private String Ms2SpectraTopic = "Spectra";
    private String PrecursorSpectraTopic = "Precursor";

    private boolean useGPU = true;

    private PaserConstants() {

    }

    private void init() {

        File propertyFile = new File("paser.properties");

        try (InputStream input = new FileInputStream(propertyFile.getCanonicalPath());) {

            Properties prop = new Properties();
            prop.load(input);
            this.kafkaIP = prop.getProperty("kafka_ip");
            this.NumberOfThreadsForGPUSearch = Integer.parseInt(prop.getProperty("num_threads"));
            this.dataOutPath =  prop.getProperty("output_path");
            this.pathToPtxFile = prop.getProperty("ptx_file_path");
            this.pathToSearchXmlFile = prop.getProperty("search_xml_path");
            this.useGPU = (prop.getProperty("use_gpu").equals("true"));

        } catch (IOException ex) {
            ex.printStackTrace();
        }

        System.out.println(toString());
    }

    public static PaserConstants getInstance() {

        if (instance == null) {

            instance = new PaserConstants();

            instance.init();
        }
        return instance;
    }

    public String getKafkaIP() {
        return kafkaIP;
    }

    public int getNumberOfThreadsForGPUSearch() {
        return NumberOfThreadsForGPUSearch;
    }

    public String getDataOutPath() {
        return FilenameUtils.separatorsToSystem(dataOutPath);
    }

    public String getInitStartKey() {
        return InitStartKey;
    }

    public String getInitEndKey() {
        return InitEndKey;
    }

    public String getInitCompletedKey() {
        return InitCompletedKey;
    }

    public String getInitFailedKey() {
        return InitFailedKey;
    }

    public String getInitializationTopic() {
        return InitializationTopic;
    }

    public String getMs2SpectraTopic() {
        return Ms2SpectraTopic;
    }

    public String getPrecursorSpectraTopic() {
        return PrecursorSpectraTopic;
    }

    public String getPathToPtxFile() {
        return FilenameUtils.separatorsToSystem(pathToPtxFile);
    }

    public String getPathToSearchXmlFile() {
        return FilenameUtils.separatorsToSystem(pathToSearchXmlFile);
    }

    public Boolean getUseGpu() {
        return useGPU;
    }


    @Override
    public String toString() {
        return "PaserConstants{" +
                "kafkaIP='" + kafkaIP + '\'' +
                ", NumberOfThreadsForGPUSearch=" + NumberOfThreadsForGPUSearch +
                ", dataOutPath='" + dataOutPath + '\'' +
                ", InitStartKey='" + InitStartKey + '\'' +
                ", InitEndKey='" + InitEndKey + '\'' +
                ", InitCompletedKey='" + InitCompletedKey + '\'' +
                ", InitializationTopic='" + InitializationTopic + '\'' +
                ", Ms2SpectraTopic='" + Ms2SpectraTopic + '\'' +
                '}';
    }
}
