
# verify that the Python API client received the same MS/MS spectra as would be obtained from the
# recorded raw data using the TDF SDK

import sqlite3, os, struct
from bdal.io import timsdata

analysis_dir = "D:\\Data\\Acq000047.d"

tdf = timsdata.TimsData(analysis_dir)
tdf.conn.execute("ATTACH '{0}' AS msms".format(os.path.join(analysis_dir, "online-msms-spectra.sqlite")))

rows = tdf.conn.execute("""
SELECT p.Id, s.ROWID FROM Precursors p
JOIN Frames f ON f.Id = p.Parent
JOIN msms.MsMsSpectra s
   ON ABS(p.AverageMz - s.PrecursorAverageMz) < 1e-8
  AND ABS(p.ScanNumber - s.PrecursorScanNumber) < 1e-8
  AND ABS(f.Time - s.Time) < 2.0
ORDER BY p.Id
""")

analyzed_ids_1 = set()
analyzed_ids_2 = set()
num_critical = 0

def readMsMsFromOurSqlite(rowid):
    row = tdf.conn.execute("SELECT MzDoubles, AreaFloats FROM MsMsSpectra WHERE ROWID={0}".format(rowid)).fetchone()
    num_entries = int(len(row[0]) / 8)
    return (
        struct.unpack('<' + 'd' * num_entries, row[0]),
        struct.unpack('<' + 'f' * num_entries, row[1])
        )

for row in rows:
    print(row[0], row[1])
    analyzed_ids_1.add(row[0])
    analyzed_ids_2.add(row[1])

    ( x1, y1 ) = tdf.readPasefMsMs([ row[0] ])[row[0]]
    #print(x1[:10])
    #print(y1[-10:])
    ( x2, y2 ) = readMsMsFromOurSqlite(row[1])
    #print(x2[:10])
    #print(y2[-10:])

    i = 0
    j = 0
    while i < len(x1) and j < len(x2):
        if abs(x1[i] - x2[j]) < 1e-12*x1[i] and y1[i] == y2[j]:
            i = i + 1
            j = j + 1
            continue

        def surrounding(list, idx, pad):
            res = []
            for i in range(idx-pad, idx+pad+1):
                if i >= 0 and i < len(list):
                    res.append(list[i])
                else:
                    res.append("---")
            return res

        print("Mismatch at indices {0} / {1}".format(i, j))
        print(surrounding(x1, i, 2))
        print(surrounding(y1, i, 2))
        print(surrounding(x2, j, 2))
        print(surrounding(y2, j, 2))
        asdf

rows = tdf.conn.execute("SELECT Id FROM Precursors").fetchall()
print("In TDF, but not received by Python:", analyzed_ids_1 ^ set([ row[0] for row in rows ]))

rows = tdf.conn.execute("SELECT ROWID FROM msms.MsMsSpectra").fetchall()
print("Received by Python, but not in TDF:", analyzed_ids_2 ^ set([ row[0] for row in rows ]))
